
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Dietrich Zerr
 */
public class Problemgenerator {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if (args.length < 1) {
            printUsageErrorAndExit();
        }

        int numberOfInstances = 1;
        boolean firstArgIsNumber;
        Problem problemInstance = null;
        String className = null;
        String problemFileName = null;
        String domainFileName = null;
        int problemProperties[] = new int[0];
        
        try {
            numberOfInstances = Integer.parseInt(args[0]);
            firstArgIsNumber = true;
            if (args.length < 2) {
                printUsageErrorAndExit();
            }
            if (numberOfInstances < 1) {
                numberOfInstances = 1;
                System.out.println("Number of instances can't be smaller then 1. Has been set to 1.");
            }
            className = args[1];
        } catch (NumberFormatException ex) {
            firstArgIsNumber = false;
            className = args[0];
        }

        try {
            problemInstance = Problem.class.cast(Class.forName(className).newInstance());
            if (problemInstance.getProblemFileName() != null) {
                problemFileName = problemInstance.getProblemFileName();
            } else {
                problemFileName = className + "-";
            }
            if (problemInstance.getDomainFileName() != null) {
                domainFileName = problemInstance.getDomainFileName();
            } else {
                domainFileName = className + "-";
            }
        } catch (Exception ex) {
            System.out.println(ex);
            System.out.println("Failed to instantiate " + className + " as a problem.");
            System.exit(-1);
        }
        
        if (firstArgIsNumber && args.length > 2) {
            problemProperties = new int[args.length - 2];
            for (int i = 0; i + 2 < args.length; i++) {
                problemProperties[i] = Integer.parseInt(args[i + 2]);
            }
        }
        if (!firstArgIsNumber && args.length > 1) {
            problemProperties = new int[args.length - 1];
            for (int i = 0; i + 1 < args.length; i++) {
                problemProperties[i] = Integer.parseInt(args[i + 1]);
            }
        }

        Properties properties = new Properties();

        try {
            properties.load(new FileInputStream("Problemgenerator.properties"));
        } catch (IOException ex) {
            try {
                properties.load(new FileInputStream("../Problemgenerator.properties"));
            } catch (IOException ex2) {
                System.out.println(ex);
                System.exit(-1);
            }
        }
        
        File outputDir = new File(properties.getProperty("output-directory"));
        outputDir.mkdirs();
        File domainFile = new File(outputDir, domainFileName 
                + properties.getProperty("domain-file-suffix") 
                + properties.getProperty("file-ending"));
        try {
                // Already existing files with the same name will be overwirtten!
                domainFile.createNewFile();
                FileWriter fw = new FileWriter(domainFile.getAbsoluteFile());
                BufferedWriter bw = new BufferedWriter(fw);
                bw.write(problemInstance.getPDDLDomain(problemProperties));
                bw.close();
            } catch (IOException ex) {
                Logger.getLogger(Problemgenerator.class.getName()).log(Level.SEVERE, null, ex);
            } catch (IllegalArgumentException ex) {
                System.out.println(ex);
                System.exit(-1);
            }
        for (int instanceNumber = 1; instanceNumber <= numberOfInstances; instanceNumber++) {
            File problemFile = new File(outputDir, problemFileName 
                    + properties.getProperty("problem-file-suffix") + instanceNumber
                    + properties.getProperty("file-ending"));
            try {
                // Already existing files with the same name will be overwirtten!
                problemFile.createNewFile();
                FileWriter fw = new FileWriter(problemFile.getAbsoluteFile());
                BufferedWriter bw = new BufferedWriter(fw);
                bw.write(problemInstance.getPDDLProblem(problemProperties, instanceNumber));
                bw.close();
            } catch (IOException ex) {
                Logger.getLogger(Problemgenerator.class.getName()).log(Level.SEVERE, null, ex);
            } catch (IllegalArgumentException ex) {
                System.out.println(ex);
                System.exit(-1);
            }
        }

        System.out.println(numberOfInstances + " PDDL problems generated at " + outputDir.getAbsolutePath());

    }

    private static void printUsageErrorAndExit() {
        System.out.println("Invalid program call.\n"
                + "Use \"Problemgenerator [<number of problems to generate>] (package.)ProblemClassName [<property integers>]*\"\n"
                + "e.g. \"Problemgenerator 5 Sokoban 3 10 10 4\"");
        System.exit(-1);
    }

}
